 ///////////////////////////////////////////////////////////////////////////////
 //
 // Project:  Aurora 64B/66B
 // Company:  Xilinx
 //
 //
 //
 // (c) Copyright 2008 - 2009 Xilinx, Inc. All rights reserved.
 //
 // This file contains confidential and proprietary information
 // of Xilinx, Inc. and is protected under U.S. and
 // international copyright and other intellectual property
 // laws.
 //
 // DISCLAIMER
 // This disclaimer is not a license and does not grant any
 // rights to the materials distributed herewith. Except as
 // otherwise provided in a valid license issued to you by
 // Xilinx, and to the maximum extent permitted by applicable
 // law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
 // WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
 // AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
 // BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
 // INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
 // (2) Xilinx shall not be liable (whether in contract or tort,
 // including negligence, or under any other theory of
 // liability) for any loss or damage of any kind or nature
 // related to, arising under or in connection with these
 // materials, including for any direct, or any indirect,
 // special, incidental, or consequential loss or damage
 // (including loss of data, profits, goodwill, or any type of
 // loss or damage suffered as a result of any action brought
 // by a third party) even if such damage or loss was
 // reasonably foreseeable or Xilinx had been advised of the
 // possibility of the same.
 //
 // CRITICAL APPLICATIONS
 // Xilinx products are not designed or intended to be fail-
 // safe, or for use in any application requiring fail-safe
 // performance, such as life-support or safety devices or
 // systems, Class III medical devices, nuclear facilities,
 // applications related to the deployment of airbags, or any
 // other applications that could lead to death, personal
 // injury, or severe property or environmental damage
 // (individually and collectively, "Critical
 // Applications"). Customer assumes the sole risk and
 // liability of any use of Xilinx products in Critical
 // Applications, subject only to applicable laws and
 // regulations governing limitations on product liability.
 //
 // THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
 // PART OF THIS FILE AT ALL TIMES.
 
 //
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  g1_ipcat_aur_RX_LL_NFC
 //
 //
 //  Description: the g1_ipcat_aur_RX_LL_NFC module detects, decodes and executes NFC messages
 //               from the channel partner.  When a message is recieved, the module
 //               signals the g1_ipcat_aur_TX_LL module that idles are required until the number
 //               of idles the g1_ipcat_aur_TX_LL module sends are enough to fulfil the request.
 //
 //
 
 `timescale 1 ns / 10 ps
 
(* DowngradeIPIdentifiedWarnings="yes" *)
 module g1_ipcat_aur_RX_LL_NFC
 (
     // Aurora Lane Interface
     RX_SNF,
     RX_XOFF,
     RX_FC_NB,
 
     // TX_LL Interface
     DECREMENT_NFC,
 
     TX_WAIT,
 
     // Global Logic Interface
     CHANNEL_UP,
 
 
     // USER Interface
     USER_CLK
 
  );
 
 `define DLY #1
 
 //***********************************Port Declarations*******************************
 
     // Aurora Lane Interface
       input      [0:3]      RX_SNF; 
       input      [0:31]     RX_FC_NB; 
       input      [0:3]      RX_XOFF; 
 
 
     // TX_LL Interface
       input                 DECREMENT_NFC; 
 
       output                TX_WAIT; 
 
 
     // Global Logic Interface
       input                 CHANNEL_UP; 
 
 
     // USER Interface
       input                 USER_CLK; 
 
 
 //**************************Internal Register Declarations****************************
 
       reg        [0:7]      nfc_counter_r; 
       reg                   TX_WAIT; 
     
 //**************************Wire Declarations****************************   
       wire       [0:7]      rx_fc_nb_c; 
 
 //*********************************Main Body of Code**********************************
 
 
     assign  rx_fc_nb_c =  RX_SNF[0] ? RX_FC_NB[0:7]:
 RX_SNF[1] ? RX_FC_NB[8:15]:
 RX_SNF[2] ? RX_FC_NB[16:23]:
 RX_SNF[3] ? RX_FC_NB[24:31]:
 8'b0;
     //________________  Use the RX_FC_NB code to set the counter_______________
 
 
     // We use a counter to keep track of the number of dead cycles we must produce to
     // satisfy the NFC request from the Channel Partner.  Note we *decrement* nfc_counter
     // when decrement NFC is asserted.  This is because the nfc counter uses the difference
     // between the max value and the current value to determine how many cycles to demand
     // a pause.  This allows us to use the carry chain more effectively to save LUTS, and
     // gives us a registered output from the counter.
     
     always @(posedge USER_CLK)
         if (!CHANNEL_UP)          nfc_counter_r  <=  `DLY    8'h00;
         else if (|RX_SNF)         nfc_counter_r  <=  `DLY    rx_fc_nb_c;
         else if (|RX_XOFF)        nfc_counter_r  <=  `DLY    8'h00;
         else if ((DECREMENT_NFC==1'b1) && (nfc_counter_r!=8'h00) && ((|RX_XOFF)==1'b0))
                                   nfc_counter_r  <=  `DLY    nfc_counter_r - 8'h01;
     // The TXWAIT signal comes from the MSBit of the counter.  We wait whenever the counter
     // is not at max value.
     always @(posedge USER_CLK)
         if (!CHANNEL_UP)        TX_WAIT         <=  `DLY    1'b0;
         else if  (|RX_XOFF) 
                                 TX_WAIT         <=  `DLY    1'b1;
         else                    TX_WAIT         <=  `DLY    ((~TX_WAIT && (nfc_counter_r > 0)) || ((TX_WAIT && ~(DECREMENT_NFC && (nfc_counter_r == 1'b1 ||nfc_counter_r ==1'b0) ))));
 
 endmodule
 
